@@ -137,9 +137,9 @@ class MeasurementTree:
 
 
     def construct_tree(self):
-        self.definition = {i: self.convert_to_dict(k) for (i, k) in self.f['scan_definition'].items()}
-        self.devices = {i: self.convert_to_dict(k, truncate=True) for (i, k) in self.f['devices'].items()}
-        self.labbook = {i: self.convert_to_dict(k) for (i, k) in self.f['labbook'].items()}
+        self.definition = {self.check_for_sp_char(i): self.convert_to_dict(k) for (i, k) in self.f['scan_definition'].items()}
+        self.devices = {self.check_for_sp_char(i): self.convert_to_dict(k, truncate=True) for (i, k) in self.f['devices'].items()}
+        self.labbook = {self.check_for_sp_char(i): self.convert_to_dict(k) for (i, k) in self.f['labbook'].items()}
         self.logs = self.convert_to_dict(self.f['measurement/log'])
         self.eLab_meta = self.get_elab_attrs()
         """
@@ -316,7 +316,7 @@ class MeasurementTree:
                 u['control name'] = key_control_name
                 control_keys.append(key_control_name)
                 u['units'] = unit
-            print(u)
+            # print(u)
         # print([u['units'] for u in self.definition.values() if 'units' in u])
 
         all_indicators = []
@@ -374,15 +374,18 @@ class MeasurementTree:
                 # if self.definition[parent_row]['function'] in ['scalar control']:
 
                 row_data = None
+                roi = None
                 try:
                     row_data = parent.get_data()
                     # Add shape of dimension to shape list
                     shape.append(row_data.shape[0])
+                    roi = [len(row_data)]
                     # Add control name as key to the coords dict. Assign data to that key.
                 except KeyError as err:
                     if self.definition[parent_row]['function'] == 'internal - repetitions':
                         print('Repetitions. Generating incrementing as coords.')
                         rep = int(self.definition[parent_row]['repetitions'])
+                        roi = [rep]
                         shape.append(rep)
                         row_data = np.arange(rep)
                     elif self.definition[parent_row]['function'] == 'scalar control':
@@ -410,7 +413,8 @@ class MeasurementTree:
                 if row_data is not None:
                     coords[control_name] = row_data
                     units.append(unit)
-                    segments.append(roi)
+                    if roi is not None:
+                        segments.append(roi)
                 parent_row = parent.parent_row
                 parent = parent.parent_group
 
@@ -431,7 +435,7 @@ class MeasurementTree:
                     scales = self.get_scales(self.target[row][0], i)
                     coords[coord_name] = scales
                     dims.append(coord_name)
-                    segments.append(None)
+                    segments.append([len(scales)])
                     unit = metadata['unit'][i]
                     units.append(unit)
             else:
@@ -440,7 +444,7 @@ class MeasurementTree:
                 for i, dat_shape in enumerate(data_shape):
                     coord_name = self.avoid_duplicate(coord_name, coords.keys())
                     coords[coord_name] = np.arange(dat_shape)
-                    segments.append(None)
+                    segments.append(float('nan'))
                     dims.append(coord_name)
                     units.append('')
 
@@ -449,6 +453,7 @@ class MeasurementTree:
             self.shape = tuple(shape)+data_shape
             try:
                 self.data = np.reshape(self.data, self.shape)
+                print(f'Desired shape: {self.shape}, data shape: {self.data.shape}, size: {self.data.size}')
             except ValueError:
                 # TODO: Find a way to drop empty slices
                 print('Measurement not finished?')
@@ -477,6 +482,7 @@ class MeasurementTree:
                 self.array[x].attrs['segments'] = z
             self.array.attrs['units'] = indicator_unit
             all_indicators.append(self.array)
+            print(f'Segments: {segments}')
 
         self.dataset = xr.combine_by_coords(all_indicators)
 
@@ -492,10 +498,6 @@ class MeasurementTree:
         from json import dumps
         for attr, val in self.attrs.items():
             self.dataset.attrs[attr] = dumps(val)
-            # if isinstance(val, dict):
-            #     self.dataset.attrs[attr] = dumps(val)
-            # elif isinstance(val, str):
-            #     self.dataset.attrs[attr] = dumps(val)
 
     def print_metadata(self, metadata):
         """
@@ -572,15 +574,16 @@ class MeasurementTree:
             return None
 
     def get_elab_attrs(self, group: str = 'eLab') -> dict:
+        group_dict = {}
         try:
             group_dict = {key: value for key, value in self.f[group].attrs.items()}
             for key, value in group_dict.items():
                 try:
-                    group_dict[key] = value.decode('UTF-8')
-                except AttributeError:
+                    group_dict[key] = value
+                except KeyError:
                     pass
-        except AttributeError:
-            raise Warning('The eLab Group could not be found in the hdf5 file.')
+        except KeyError:
+            print('The eLab Group could not be found in the hdf5 file.')
         return group_dict
 
 
@@ -588,6 +591,7 @@ class MeasurementTree:
     def check_for_sp_char(text):
         import re
         special_char = re.compile(r"%%%(\d+)%%%")
+        text = str(text)
         res = special_char.finditer(text)
         for v in res:
             text = text.replace(v.group(), chr(int(v.group(1))))
